package com.engine.kq.cmd.report;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.engine.common.biz.AbstractCommonCommand;
import com.engine.common.entity.BizLogContext;
import com.engine.core.interceptor.CommandContext;
import com.engine.kq.biz.*;
import com.engine.xmgsecond.entity.KqFormatTotal;
import com.engine.xmgsecond.util.KqCalulateUtil;
import com.weaver.file.Prop;
import com.engine.kq.util.KQDurationCalculatorUtil;
import com.engine.kq.util.PageUidFactory;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.hrm.company.DepartmentComInfo;
import weaver.hrm.company.SubCompanyComInfo;
import weaver.hrm.job.JobTitlesComInfo;
import weaver.hrm.resource.ResourceComInfo;
import weaver.systeminfo.SystemEnv;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class GetKQReportCmd extends AbstractCommonCommand<Map<String, Object>> {

	public GetKQReportCmd(Map<String, Object> params, User user) {
		this.user = user;
		this.params = params;
	}

	@Override
	public Map<String, Object> execute(CommandContext commandContext) {
		Map<String,Object> retmap = new HashMap<String,Object>();
		RecordSet rs = new RecordSet();
		String sql = "";
		try{
			String pageUid = PageUidFactory.getHrmPageUid("KQReport");

			SubCompanyComInfo subCompanyComInfo = new SubCompanyComInfo();
			DepartmentComInfo departmentComInfo = new DepartmentComInfo();
			ResourceComInfo resourceComInfo = new ResourceComInfo();
			JobTitlesComInfo jobTitlesComInfo = new JobTitlesComInfo();
			KQLeaveRulesBiz kqLeaveRulesBiz = new KQLeaveRulesBiz();
			KQReportBiz kqReportBiz = new KQReportBiz();
			KQSettingsComInfo kqSettingsComInfo = new KQSettingsComInfo();
			String leavetype_is_on = Util.null2String(kqSettingsComInfo.getMain_val("leavetype_is_on"),"0");

			JSONObject jsonObj = JSON.parseObject(Util.null2String(params.get("data")));
			String attendanceSerial = Util.null2String(jsonObj.get("attendanceSerial"));
			String fromDate = Util.null2String(jsonObj.get("fromDate"));
			String toDate = Util.null2String(jsonObj.get("toDate"));
			String typeselect =Util.null2String(jsonObj.get("typeselect"));
			if(typeselect.length()==0)typeselect = "3";
			if(!typeselect.equals("") && !typeselect.equals("0")&& !typeselect.equals("6")){
				if(typeselect.equals("1")){
					fromDate = TimeUtil.getCurrentDateString();
					toDate = TimeUtil.getCurrentDateString();
				}else{
					fromDate = TimeUtil.getDateByOption(typeselect,"0");
					toDate = TimeUtil.getDateByOption(typeselect,"1");
				}
			}
			//人员状态
			String status = Util.null2String(jsonObj.get("status"));
			String subCompanyId = Util.null2String(jsonObj.get("subCompanyId"));
			String departmentId = Util.null2String(jsonObj.get("departmentId"));
			String resourceId = Util.null2String(jsonObj.get("resourceId"));
			String allLevel = Util.null2String(jsonObj.get("allLevel"));
			String isNoAccount = Util.null2String(jsonObj.get("isNoAccount"));
			String viewScope = Util.null2String(jsonObj.get("viewScope"));
			String isFromMyAttendance = Util.null2String(jsonObj.get("isFromMyAttendance"));//是否是来自我的考勤的请求，如果是，不加载考勤报表权限共享的限制，不然我的考勤会提示无权限
			int pageIndex = Util.getIntValue(Util.null2String(jsonObj.get("pageIndex")), 1);
			int pageSize =  KQReportBiz.getPageSize(Util.null2String(jsonObj.get("pageSize")),pageUid,user.getUID());
			boolean isKqReportHalfOpen = "1".equals(new KQSettingsComInfo().getMain_val("kq_report_half"));
			if(isKqReportHalfOpen) {
				pageSize = pageSize/2;
			}
			int count = 0;
			int pageCount = 0;
			int isHavePre = 0;
			int isHaveNext = 0;


			String rightSql = kqReportBiz.getReportRight("1",""+user.getUID(),"a");
			if(isFromMyAttendance.equals("1")){
				rightSql = "";
			}

			List<Map<String, Object>> leaveRules = kqLeaveRulesBiz.getAllLeaveRules();
			List<Object> columns = new ArrayList();
			Map<String,Object> column = null;
			List<Object> datas = new ArrayList();
			Map<String,Object> data = null;
			Map<String,Object> mapChildColumnInfo = null;
			List<Object> childColumns = null;
			KQReportFieldComInfo kqReportFieldComInfo = new KQReportFieldComInfo();
			while (kqReportFieldComInfo.next()){
				//QC3685035 考勤汇总表去除严重迟到,严重早退（+对应时长）4列
				if ("graveBeLate".equals(kqReportFieldComInfo.getFieldname())) continue;
				if ("graveLeaveEarly".equals(kqReportFieldComInfo.getFieldname())) continue;
				if ("graveBeLateMins".equals(kqReportFieldComInfo.getFieldname())) continue;
				if ("graveLeaveEarlyMins".equals(kqReportFieldComInfo.getFieldname())) continue;

				if(Util.null2String(kqReportFieldComInfo.getParentid()).length()>0)continue;
				if(kqReportFieldComInfo.getFieldname().equals("kqCalendar"))continue;
				if(KQReportFieldComInfo.cascadekey2fieldname.keySet().contains(kqReportFieldComInfo.getFieldname()))continue;
				if(!kqReportFieldComInfo.getReportType().equals("all") && !kqReportFieldComInfo.getReportType().equals("month"))continue;
				if("leave".equalsIgnoreCase(kqReportFieldComInfo.getFieldname())&&leaveRules.size()==0)continue;

				column = new HashMap();
				column.put("title", SystemEnv.getHtmlLabelNames(kqReportFieldComInfo.getFieldlabel(), user.getLanguage()));
				column.put("unit", KQReportBiz.getUnitType(kqReportFieldComInfo, user));
				column.put("dataIndex", kqReportFieldComInfo.getFieldname());
				column.put("type", kqReportFieldComInfo.getFieldname());
				column.put("key", kqReportFieldComInfo.getFieldname());
				column.put("isSystem", kqReportFieldComInfo.getIsSystem());
				mapChildColumnInfo = this.getChildColumnsInfo(kqReportFieldComInfo.getFieldname(),user);
				childColumns = (List<Object>)mapChildColumnInfo.get("childColumns");
				if(childColumns.size()>0) {//跨列width取子列的width
					column.put("rowSpan", 1);
					column.put("width", mapChildColumnInfo.get("sumChildColumnWidth"));
					column.put("children", childColumns);
				}else{
					column.put("rowSpan", 3);
					column.put("width", Util.getIntValue(kqReportFieldComInfo.getWidth()));
				}
				column.put("showDetial",kqReportFieldComInfo.getShowDetial());
				columns.add(column);
				columns.addAll(this.getCascadeKeyColumnsInfo(kqReportFieldComInfo.getCascadekey(),user));
			}

			boolean isEnd = false;
			Calendar cal = DateUtil.getCalendar();
			String today = DateUtil.getCurrentDate();
//			if(DateUtil.compDate(today, toDate)>0){//结束日期不大于今天
//				toDate = today;
//				if(DateUtil.compDate(today, fromDate)>0){//结束日期不大于今天
//          fromDate = today;
//        }
//			}

			childColumns = new ArrayList<>();
			for(String date=fromDate; !isEnd;) {
				if(date.equals(toDate)) isEnd = true;
				column = new HashMap();
				column.put("title", DateUtil.geDayOfMonth(date));
				column.put("dataIndex", date);
				column.put("key", date);
				column.put("type", date);
				column.put("rowSpan", 1);
				column.put("width", 90);
				column.put("isCalendar", 1);
				childColumns.add(column);
				cal.setTime(DateUtil.parseToDate(date));
				date = DateUtil.getDate(cal.getTime(), 1);
			}

			column = new HashMap();
			column.put("title", SystemEnv.getHtmlLabelName(386476, user.getLanguage()));
			column.put("dataIndex", "kqCalendar");
			column.put("key", "kqCalendar");
			if(childColumns.size()>0) {//跨列width取子列的width
				column.put("rowSpan", 1);
				column.put("width", childColumns.size()*90);
				column.put("children", childColumns);
			}
			columns.add(column);


			// 没有权限的人最大只支持查询本季
			Map<String, String> getRealDate = new KQReportBiz().realDate(fromDate, toDate, user, "1");
			fromDate = getRealDate.get("fromDate");
			toDate = getRealDate.get("toDate");

			String forgotBeginWorkCheck_field = " sum(b.forgotBeginWorkCheck) ";

			if(rs.getDBType().equalsIgnoreCase("oracle")) {
				forgotBeginWorkCheck_field = " sum(nvl(b.forgotBeginWorkCheck,0))  ";
			}else if((rs.getDBType()).equalsIgnoreCase("mysql")){
				forgotBeginWorkCheck_field = " sum(ifnull(b.forgotBeginWorkCheck,0)) ";
			}else {
				forgotBeginWorkCheck_field = " sum(isnull(b.forgotBeginWorkCheck,0)) ";
			}

			Map<String,Object> definedFieldInfo = new KQFormatBiz().getDefinedField();
			String definedFieldSum = Util.null2String(definedFieldInfo.get("definedFieldSum"));

			String backFields = " a.id,a.lastname,a.workcode,a.dsporder,b.resourceid,a.subcompanyid1 as subcompanyid,a.departmentid,a.jobtitle,b.groupid," +
					" sum(b.workdays) as workdays,sum(b.workMins) as workMins,sum(b.attendancedays) as attendancedays," +
					" sum(b.attendanceMins) as attendanceMins,sum(b.beLate) as beLate,sum(b.beLateMins) as beLateMins, " +
					" sum(b.graveBeLate) as graveBeLate, sum(b.graveBeLateMins) as graveBeLateMins,sum(b.leaveEearly) as leaveEearly," +
					" sum(b.leaveEarlyMins) as leaveEarlyMins, sum(b.graveLeaveEarly) as graveLeaveEarly, " +
					" sum(b.graveLeaveEarlyMins) as graveLeaveEarlyMins,sum(b.absenteeism) as absenteeism, " +
					" sum(b.signdays) as signdays,sum(b.signmins) as signmins, "+
					" sum(b.absenteeismMins) as absenteeismMins, sum(b.forgotCheck)+"+forgotBeginWorkCheck_field+" as forgotCheck "+(definedFieldSum.length()>0?","+definedFieldSum+"":"");

			if(rs.getDBType().equals("oracle")){
				backFields = 	"/*+ index(kq_format_total IDX_KQ_FORMAT_TOTAL_KQDATE) */ "+backFields;
			}
			String sqlFrom = " from hrmresource a, kq_format_total b where a.id= b.resourceid and b.kqdate >='"+fromDate+"' and b.kqdate <='"+toDate+"'";
			String sqlWhere = rightSql;
			String groupBy = " group by a.id,a.lastname,a.workcode,a.dsporder,b.resourceid,a.subcompanyid1,a.departmentid,a.jobtitle,b.groupid ";
			List<List> paramLists = new ArrayList<>();
			List<Object> paramList = null;
			if(subCompanyId.length()>0){
				sqlWhere +=" and a.subcompanyid1 in("+KQReportBiz.splitParamStr(subCompanyId)+") ";
				for(String bean : subCompanyId.split(",")) {
					paramList = new ArrayList<>();
					paramList.add(bean);
					paramLists.add(paramList);
				}
			}

			if(departmentId.length()>0){
				sqlWhere +=" and a.departmentid in("+KQReportBiz.splitParamStr(departmentId)+") ";
				for(String bean : departmentId.split(",")) {
					paramList = new ArrayList<>();
					paramList.add(bean);
					paramLists.add(paramList);
				}
			}

			if(resourceId.length()>0){
				sqlWhere +=" and a.id in("+KQReportBiz.splitParamStr(resourceId)+") ";
				for(String bean : resourceId.split(",")) {
					paramList = new ArrayList<>();
					paramList.add(bean);
					paramLists.add(paramList);
				}
			}


			if(viewScope.equals("4")){//我的下属
				if(allLevel.equals("1")){//所有下属
					sqlWhere+=" and a.managerstr like '%,"+user.getUID()+",%'";
				}else{
					sqlWhere+=" and a.managerid="+user.getUID();//直接下属
				}
			}
			if (!"1".equals(isNoAccount)) {
				sqlWhere += " and a.loginid is not null "+(rs.getDBType().equals("oracle")?"":" and a.loginid<>'' ");
			}

			if(status.length()>0){
				if (!status.equals("8") && !status.equals("9")) {
					sqlWhere += " and a.status = ?";
					paramList = new ArrayList<>();
					paramList.add(status);
					paramLists.add(paramList);
				}else if (status.equals("8")) {
					sqlWhere += " and (a.status = 0 or a.status = 1 or a.status = 2 or a.status = 3) ";
				}
			}
			if("-1".equals(fromDate) || "-1".equals(toDate)) {
				sqlWhere += " and 1=2 ";
			}
			sql = " select count(*) as c from ( select 1 as c "+sqlFrom+sqlWhere+groupBy+") t";
			rs.executeQuery(sql, paramLists);
			if (rs.next()){
				count = rs.getInt("c");
			}

			if (count <= 0) {
				pageCount = 0;
			}

			pageCount = count / pageSize + ((count % pageSize > 0) ? 1 : 0);

			isHaveNext = (pageIndex + 1 <= pageCount) ? 1 : 0;

			isHavePre = (pageIndex - 1 >= 1) ? 1 : 0;

			String orderBy = " order by t.dsporder asc, t.lastname asc ";
			String descOrderBy = " order by t.dsporder desc, t.lastname desc ";

			//默认排序设置 start，有性能问题，先取消，后面再看看有没有好的方式
//			String orderBySql = "select * from kq_report_order where userId=? and sort=1 order by orders";
//			rs.executeQuery(orderBySql, user.getUID());
//			if (rs.getCounts() <= 0) {
//				orderBySql = "select * from kq_report_order where userId=0 and sort=1 order by orders";
//				rs.executeQuery(orderBySql);
//			}
//			while (rs.next()) {
//				String dataIndex = rs.getString("dataIndex");
//				String ascOrDesc = rs.getString("ascOrDesc");
//				String ascOrDesc1 = (ascOrDesc.equals("")||ascOrDesc.equals("asc"))?"desc":"asc";
//				if (dataIndex.equals("organization")) {
//					orderBy += ",showOrderOfDeptTree " + ascOrDesc + ",dept_id " + ascOrDesc;
//					descOrderBy += ",showOrderOfDeptTree " + ascOrDesc1 + ",dept_id " + ascOrDesc1;
//				}
//				if (dataIndex.equalsIgnoreCase("dspOrder") || dataIndex.equalsIgnoreCase("lastName")) {
//					orderBy += "," + dataIndex + " " + ascOrDesc + ",id " + ascOrDesc;
//					descOrderBy += "," + dataIndex + " " + ascOrDesc1 + ",id " + ascOrDesc1;
//				} else if (dataIndex.equalsIgnoreCase("deptShowOrder") || dataIndex.equalsIgnoreCase("deptName")) {
//					orderBy += "," + dataIndex + " " + ascOrDesc + ",dept_id " + ascOrDesc;
//					descOrderBy += "," + dataIndex + " " + ascOrDesc1 + ",dept_id " + ascOrDesc1;
//				} else if (dataIndex.equalsIgnoreCase("subcomShowOrder") || dataIndex.equalsIgnoreCase("subcomName")) {
//					orderBy += "," + dataIndex + " " + ascOrDesc + ",subcom_id " + ascOrDesc;
//					descOrderBy += "," + dataIndex + " " + ascOrDesc1 + ",subcom_id " + ascOrDesc1;
//				}
//			}
//			orderBy = orderBy.startsWith(",") ? orderBy.substring(1) : orderBy;
//			descOrderBy = descOrderBy.startsWith(",") ? descOrderBy.substring(1) : descOrderBy;
//			orderBy = orderBy.equals("") ? " t.dspOrder,t.id " : orderBy;
//			descOrderBy = descOrderBy.equals("") ? " t.dspOrder,t.id " : descOrderBy;
//			orderBy = "order by "+orderBy;

			sql = backFields + sqlFrom  + sqlWhere + groupBy;

			if (pageIndex > 0 && pageSize > 0) {
				if (rs.getDBType().equals("oracle")) {
					sql = " select * from (select " + sql+") t "+orderBy;
					sql = "select * from ( select row_.*, rownum rownum_ from ( " + sql + " ) row_ where rownum <= "
							+ (pageIndex * pageSize) + ") where rownum_ > " + ((pageIndex - 1) * pageSize);
				} else if (rs.getDBType().equals("mysql")) {
					sql = " select * from (select " + sql+") t "+orderBy;
					sql = "select t1.* from (" + sql + ") t1 limit " + ((pageIndex - 1) * pageSize) + "," + pageSize;
				}
				else if (rs.getDBType().equals("postgresql")) {
					sql = " select * from (select " + sql+") t "+orderBy;
					sql = "select t1.* from (" + sql + ") t1 limit " +pageSize + " offset " + ((pageIndex - 1) * pageSize);
				}
				else {
					orderBy = " order by dsporder asc, lastname asc ";
					descOrderBy = " order by dsporder desc, lastname desc ";
					if (pageIndex > 1) {
						int topSize = pageSize;
						if (pageSize * pageIndex > count) {
							topSize = count - (pageSize * (pageIndex - 1));
						}
						sql = " select top " + topSize + " * from ( select top  " + topSize + " * from ( select top "
								+ (pageIndex * pageSize) + sql + orderBy+ " ) tbltemp1 " + descOrderBy + ") tbltemp2 " + orderBy;
					} else {
						sql = " select top " + pageSize + sql+orderBy;
					}
				}
			} else {
				sql = " select " + sql;
			}


			// #1475814-概述：满足考勤报分部部门显示及导出时显示全路径
			String fullPathMainKey = "show_full_path";
			String isShowFullPath = Util.null2String(kqSettingsComInfo.getMain_val(fullPathMainKey),"0");

			String show_card_source = Util.null2String(kqSettingsComInfo.getMain_val("show_card_source"),"0");//是否显示打卡数据，以及打卡数据来源
			params.put("show_card_source",show_card_source);
			KQOvertimeRulesBiz kqOvertimeRulesBiz = new KQOvertimeRulesBiz();
			int uintType = kqOvertimeRulesBiz.getMinimumUnit();//当前加班单位
			double hoursToDay = kqOvertimeRulesBiz.getHoursToDay();//当前天跟小时计算关系
			params.put("uintType",uintType);
			params.put("hoursToDay",hoursToDay);
			Map<String,Object> flowData = kqReportBiz.getFlowData(params,user);

			rs.executeQuery(sql, paramLists);
			while (rs.next()) {
				data = new HashMap<>();
				kqReportFieldComInfo.setTofirstRow();
				String id = rs.getString("id");
				data.put("resourceId",id);
				while (kqReportFieldComInfo.next()){
					if(!Util.null2String(kqReportFieldComInfo.getIsdataColumn()).equals("1"))continue;
					if(!kqReportFieldComInfo.getReportType().equals("all") && !kqReportFieldComInfo.getReportType().equals("month"))continue;
					if("leave".equalsIgnoreCase(kqReportFieldComInfo.getFieldname())&&leaveRules.size()==0){
						continue;
					}
					String fieldName = kqReportFieldComInfo.getFieldname();
					String fieldValue = "";
					if(fieldName.equals("subcompany")){
						String tmpSubcompanyId = Util.null2String(rs.getString("subcompanyid"));
						if(tmpSubcompanyId.length()==0){
							tmpSubcompanyId =  Util.null2String(resourceComInfo.getSubCompanyID(id));
						}
						data.put("subcompanyId",tmpSubcompanyId);

						fieldValue = "1".equals(isShowFullPath) ?
								SubCompanyComInfo.getSubcompanyRealPath(tmpSubcompanyId, "/", "0") :
								subCompanyComInfo.getSubCompanyname(tmpSubcompanyId);

						// fieldValue = subCompanyComInfo.getSubCompanyname(tmpSubcompanyId);
					}else if(fieldName.equals("department")){
						String tmpDepartmentId = Util.null2String(rs.getString("departmentid"));
						if(tmpDepartmentId.length()==0){
							tmpDepartmentId =  Util.null2String(resourceComInfo.getDepartmentID(id));
						}
						data.put("departmentId",tmpDepartmentId);

						fieldValue = "1".equals(isShowFullPath) ?
								departmentComInfo.getDepartmentRealPath(tmpDepartmentId, "/", "0") :
								departmentComInfo.getDepartmentname(tmpDepartmentId);

						// fieldValue = departmentComInfo.getDepartmentname(tmpDepartmentId);
					}else if(fieldName.equals("jobtitle")){
						String tmpJobtitleId = Util.null2String(rs.getString("jobtitle"));
						if(tmpJobtitleId.length()==0){
							tmpJobtitleId =  Util.null2String(resourceComInfo.getJobTitle(id));
						}
						data.put("jobtitleId",tmpJobtitleId);
						fieldValue = jobTitlesComInfo.getJobTitlesname(tmpJobtitleId);
					}else if("kqGroup".equals(fieldName)){
						//QC3685035 考勤月度报表增加所属所属考勤组
						KQGroupComInfo kqGroupComInfo = new KQGroupComInfo();
						KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
						String groupId = kqGroupMemberComInfo.getKQGroupId(id, toDate);
						fieldValue = kqGroupComInfo.getGroupname(groupId);
//						fieldValue = kqGroupComInfo.getGroupname(Util.null2String(rs.getString("groupid")));
					}else if("absenteeismDays".equals(fieldName)){
						//QC3685035 考勤月度报表增加缺勤天数
						List<KqFormatTotal> kqFormatTotalList = KqCalulateUtil.abnormalAttendance(id, fromDate, toDate);
						fieldValue = String.format("%.1f", kqFormatTotalList.stream().mapToDouble(KqFormatTotal::getAbsenceDays).sum());
					}else if("nigntWork".equals(fieldName)){
						//QC3685035 考勤月度报表增加夜班次数
						fieldValue = String.valueOf(KqCalulateUtil.nightWorkSums(id,fromDate,toDate));
					}else if(fieldName.equals("rlzqqxs")){
						String rlzqqxsTime = getXcqts(fromDate,id);
						fieldValue = rlzqqxsTime;
					}else if(fieldName.equals("attendanceSerial")){
						List<String> serialIds = null;
						if(attendanceSerial.length()>0){
							serialIds = Util.splitString2List(attendanceSerial,",");
						}
						for(int i=0;serialIds!=null&&i<serialIds.size();i++){
							data.put(serialIds.get(i), kqReportBiz.getSerialCount(id,fromDate,toDate,serialIds.get(i)));
						}
					}else if(kqReportFieldComInfo.getParentid().equals("overtime")||kqReportFieldComInfo.getParentid().equals("overtime_nonleave")
							||kqReportFieldComInfo.getParentid().equals("overtime_4leave")||fieldName.equals("businessLeave") || fieldName.equals("officialBusiness")){
						if(fieldName.equals("overtimeTotal")){
							double workingDayOvertime_4leave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|workingDayOvertime_4leave")));
							workingDayOvertime_4leave = workingDayOvertime_4leave<0?0:workingDayOvertime_4leave;
							double restDayOvertime_4leave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|restDayOvertime_4leave")));
							restDayOvertime_4leave = restDayOvertime_4leave<0?0:restDayOvertime_4leave;
							double holidayOvertime_4leave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|holidayOvertime_4leave")));
							holidayOvertime_4leave = holidayOvertime_4leave<0?0:holidayOvertime_4leave;

							double workingDayOvertime_nonleave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|workingDayOvertime_nonleave")));
							workingDayOvertime_nonleave = workingDayOvertime_nonleave<0?0:workingDayOvertime_nonleave;
							double restDayOvertime_nonleave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|restDayOvertime_nonleave")));
							restDayOvertime_nonleave = restDayOvertime_nonleave<0?0:restDayOvertime_nonleave;
							double holidayOvertime_nonleave = Util.getDoubleValue(Util.null2String(flowData.get(id+"|holidayOvertime_nonleave")));
							holidayOvertime_nonleave = holidayOvertime_nonleave<0?0:holidayOvertime_nonleave;

							fieldValue = KQDurationCalculatorUtil.getDurationRound(String.valueOf(workingDayOvertime_4leave+restDayOvertime_4leave+holidayOvertime_4leave+
									workingDayOvertime_nonleave+restDayOvertime_nonleave+holidayOvertime_nonleave));
						}else if(fieldName.equals("businessLeave") || fieldName.equals("officialBusiness")){
							String businessLeaveData = Util.null2s(Util.null2String(flowData.get(id+"|"+fieldName)),"0.0");
							String backType = fieldName+"_back";
							String businessLeavebackData = Util.null2s(Util.null2String(flowData.get(id+"|"+backType)),"0.0");
							String businessLeave = "";
							try{
								//以防止出现精度问题
								if(businessLeaveData.length() == 0){
									businessLeaveData = "0.0";
								}
								if(businessLeavebackData.length() == 0){
									businessLeavebackData = "0.0";
								}
								BigDecimal b_businessLeaveData = new BigDecimal(businessLeaveData);
								BigDecimal b_businessLeavebackData = new BigDecimal(businessLeavebackData);
								businessLeave = b_businessLeaveData.subtract(b_businessLeavebackData).toString();
								if(Util.getDoubleValue(businessLeave, -1) < 0){
									businessLeave = "0.0";
								}
							}catch (Exception e){
							}
							fieldValue = KQDurationCalculatorUtil.getDurationRound(businessLeave);
						}else{
							fieldValue = KQDurationCalculatorUtil.getDurationRound(Util.null2String(flowData.get(id+"|"+fieldName)));
						}
					} else {
						fieldValue = Util.null2String(rs.getString(fieldName));
						if(Util.null2String(kqReportFieldComInfo.getUnittype()).length()>0) {
							if(fieldValue.length() == 0){
								fieldValue="0";
							}else{
								if (kqReportFieldComInfo.getUnittype().equals("2")) {
									fieldValue = KQDurationCalculatorUtil.getDurationRound(("" + (Util.getDoubleValue(fieldValue) / 60.0)));
								}
							}
						}
					}
					data.put(fieldName,fieldValue);
				}

				//请假
				List<Map<String, Object>> allLeaveRules = kqLeaveRulesBiz.getAllLeaveRules();
				if(("1").equals(leavetype_is_on)) {
					KQLeaveRulesComInfo kqLeaveRulesComInfo = new KQLeaveRulesComInfo();
					for (int i = allLeaveRules.size() - 1; i >= 0; i--) {
						Map<String, Object> leaveRule = allLeaveRules.get(i);
						String leaveId = Util.null2String(leaveRule.get("id"));
						boolean flag = KQLeaveRulesBiz.filterSubLeaveRule(kqLeaveRulesComInfo, user, leaveId);
						if (flag) {
							allLeaveRules.remove(i);
						}
					}
				}
				Map<String, Object> leaveRule = null;
				for(int i=0;allLeaveRules!=null&&i<allLeaveRules.size();i++){
					leaveRule = (Map<String, Object>)allLeaveRules.get(i);
					String flowType = Util.null2String("leaveType_"+leaveRule.get("id"));
					String leaveData = Util.null2String(flowData.get(id+"|"+flowType));
					String flowLeaveBackType = Util.null2String("leavebackType_"+leaveRule.get("id"));
					String leavebackData = Util.null2s(Util.null2String(flowData.get(id+"|"+flowLeaveBackType)),"0.0");
					String b_flowLeaveData = "";
					String flowLeaveData = "";
					try{
						//以防止出现精度问题
						if(leaveData.length() == 0){
							leaveData = "0.0";
						}
						if(leavebackData.length() == 0){
							leavebackData = "0.0";
						}
						BigDecimal b_leaveData = new BigDecimal(leaveData);
						BigDecimal b_leavebackData = new BigDecimal(leavebackData);
						b_flowLeaveData = b_leaveData.subtract(b_leavebackData).toString();
						if(Util.getDoubleValue(b_flowLeaveData, -1) < 0){
							b_flowLeaveData = "0.0";
						}
					}catch (Exception e){
						writeLog("GetKQReportCmd:leaveData"+leaveData+":leavebackData:"+leavebackData+":"+e);
					}

					//考虑下冻结的数据
					if(b_flowLeaveData.length() > 0){
						flowLeaveData = KQDurationCalculatorUtil.getDurationRound(b_flowLeaveData);
					}else{
						flowLeaveData = KQDurationCalculatorUtil.getDurationRound(Util.null2String(Util.getDoubleValue(leaveData,0.0)-Util.getDoubleValue(leavebackData,0.0)));
					}
					data.put(flowType,flowLeaveData);
				}

				Map<String,Object> amData = Maps.newHashMap();
				Map<String,Object> pmData = Maps.newHashMap();
				if(isKqReportHalfOpen) {
					amData.putAll(data);
					pmData.putAll(data);
				}

				Map<String,Object> detialDatas = kqReportBiz.getDetialDatas(id,fromDate,toDate,user,flowData,false,uintType,show_card_source);
//        new KQLog().info("id:"+id+":detialDatas:"+detialDatas);
				String user_endDate = Util.null2String(resourceComInfo.getEndDate(id));
				String user_status = Util.null2String(resourceComInfo.getStatus(id));
				String begindate = Util.null2String(resourceComInfo.getCreatedate(id)).trim();
				String companyStartDate = Util.null2String(resourceComInfo.getCompanyStartDate(id)).trim();
				if(companyStartDate.length()!=10){
					companyStartDate = "";
				}
				if(companyStartDate.length()>0 && companyStartDate.indexOf("-")>0){
					begindate=companyStartDate;
				}
				isEnd = false;
				for(String date=fromDate; !isEnd;) {
					if(date.equals(toDate)) isEnd = true;
					if(DateUtil.compDate(today, date)>0){
						if(isKqReportHalfOpen) {
							amData.put(date,"");
							pmData.put(date,"");
						} else {
							data.put(date,"");
						}
					}else{
						if(!(user_status.equals("0")||user_status.equals("1")||user_status.equals("2")||user_status.equals("3"))){
							//其他状态
							if(user_endDate.length()>0  && date.compareTo(user_endDate)>0){//人员合同结束日期之后
								if(isKqReportHalfOpen) {
									amData.put(date,"");
									pmData.put(date,"");
								} else {
									data.put(date,"");
								}
								cal.setTime(DateUtil.parseToDate(date));
								date = DateUtil.getDate(cal.getTime(), 1);
								continue;
							}
						}
						if(begindate.length()>0 && date.compareTo(begindate)<0 ){//人员入职日期前无需计算考勤，如果没有入职日期，已创建日期为准
							if(isKqReportHalfOpen) {
								amData.put(date,"");
								pmData.put(date,"");
							} else {
								data.put(date,"");
							}
							cal.setTime(DateUtil.parseToDate(date));
							date = DateUtil.getDate(cal.getTime(), 1);
							continue;
						}

						if(isKqReportHalfOpen) {
							Map<String,Object> tmpData = Maps.newHashMap();
							Object tmpValue = null;
							if(detialDatas.get(id+"|"+date)==null){
								tmpValue = SystemEnv.getHtmlLabelName(26593, user.getLanguage());
								data.put(date,tmpValue);
								amData.put(date,tmpValue);
								pmData.put(date,tmpValue);
							}else {
								tmpData = (Map<String, Object>) detialDatas.get(id+"|"+date);
								data.put(date,tmpData);
								if(tmpData != null){
									tmpData.put("text",Util.null2String(tmpData.get("textAM")));

									Map<String,Object> amTmpData = Maps.newHashMap();
									amTmpData.put("text",Util.null2String(tmpData.get("textAM")));
									Map<String,Object> pmTmpData = Maps.newHashMap();
									pmTmpData.put("text",Util.null2String(tmpData.get("textPM")));
									amData.put(date,amTmpData);
									pmData.put(date,pmTmpData);
								}
							}
						} else {
							data.put(date,detialDatas.get(id+"|"+date)==null?SystemEnv.getHtmlLabelName(26593, user.getLanguage()):detialDatas.get(id+"|"+date));
						}
					}
					cal.setTime(DateUtil.parseToDate(date));
					date = DateUtil.getDate(cal.getTime(), 1);
				}
				if(isKqReportHalfOpen) {
					amData.put("rowSpan", "2");
					pmData.put("rowSpan", "0");
					datas.add(amData);
					datas.add(pmData);
				} else {
					datas.add(data);
				}
			}

			List<Object> lsHolidays = KQHolidaySetBiz.getHolidaySetListByScope(""+user.getUID(),fromDate,toDate);
			retmap.put("holidays", lsHolidays);

			retmap.put("columns",columns);
			retmap.put("datas",datas);
			if(isKqReportHalfOpen) {
				retmap.put("pagesize", pageSize*2);
				retmap.put("count", count*2);
			} else {
				retmap.put("pagesize", pageSize);
				retmap.put("count", count);
			}
			retmap.put("pageindex", pageIndex);
			retmap.put("pagecount", pageCount);
			retmap.put("ishavepre", isHavePre);
			retmap.put("ishavenext", isHaveNext);
			retmap.put("isKqReportHalfOpen", isKqReportHalfOpen);
		}catch (Exception e){
			writeLog(e);
		}
		return retmap;
	}

	private Map<String,Object> getChildColumnsInfo(String parentid, User user){
		Map<String,Object> returnMap = new HashMap<>();
		List<Object> lsChildColumns = new ArrayList<>();
		Map column = null;
		int sumChildColumnWidth = 0;
		KQSettingsComInfo kqSettingsComInfo = new KQSettingsComInfo();
		String leavetype_is_on = Util.null2String(kqSettingsComInfo.getMain_val("leavetype_is_on"),"0");
		if(parentid.equals("attendanceSerial")){//考勤班次
			KQShiftManagementComInfo kqShiftManagementComInfo = new KQShiftManagementComInfo();
			JSONObject jsonObj = JSON.parseObject(Util.null2String(params.get("data")));
			List<String> serialIds = null;
			if(Util.null2String(jsonObj.get("attendanceSerial")).length()>0){
				serialIds = Util.splitString2List(Util.null2String(jsonObj.get("attendanceSerial")),",");
			}
			for(int i=0;serialIds!=null&&i<serialIds.size();i++){
				column = new HashMap();
				column.put("title", kqShiftManagementComInfo.getSerial(serialIds.get(i)));
				column.put("unit", "");
				column.put("width", 65);
				column.put("dataIndex", serialIds.get(i));
				column.put("key", serialIds.get(i));
				column.put("rowSpan", 2);
				column.put("colSpan", 1);
				sumChildColumnWidth+=65;
				lsChildColumns.add(column);
			}
		}else if(parentid.equals("leave")){
			KQLeaveRulesBiz kqLeaveRulesBiz = new KQLeaveRulesBiz();
			List<Map<String, Object>> leaveRules = kqLeaveRulesBiz.getAllLeaveRules();
			if (("1").equals(leavetype_is_on)) {
				KQLeaveRulesComInfo kqLeaveRulesComInfo = new KQLeaveRulesComInfo();
				for (int i = leaveRules.size() - 1; i >= 0; i--) {
					Map<String, Object> leaveRule = leaveRules.get(i);
					String leaveId = Util.null2String(leaveRule.get("id"));
					boolean flag = kqLeaveRulesBiz.filterSubLeaveRule(kqLeaveRulesComInfo, user, leaveId);
					if (flag) {
						leaveRules.remove(i);
					}
				}
			}
			for(int i=0;leaveRules!=null&&i<leaveRules.size();i++){
				Map<String, Object> leaveRule = leaveRules.get(i);
				String id = "leaveType_"+Util.null2String(leaveRule.get("id"));
				String name = Util.null2String(leaveRule.get("name"));
				String unitType = Util.null2String(leaveRule.get("unitType"));
				column = new HashMap();
				column.put("title", name);
				column.put("unit", KQUnitBiz.isLeaveHour(unitType) ?SystemEnv.getHtmlLabelName(391, user.getLanguage()):SystemEnv.getHtmlLabelName(1925, user.getLanguage()));
				column.put("width", 65);
				column.put("dataIndex", id);
				column.put("key", id);
				column.put("rowSpan", 2);
				column.put("colSpan", 1);
				column.put("showDetial","1");
				sumChildColumnWidth+=65;
				lsChildColumns.add(column);
			}
		}else if(parentid.equals("overtime")){
			String[] overtimeChild = {"overtime_nonleave","overtime_4leave","overtimeTotal"};
			for(int i=0;i<overtimeChild.length;i++){
				String id = overtimeChild[i];
				column = new HashMap();
				String fieldlabel = "";
				column.put("unit", "");
				if("overtime_nonleave".equalsIgnoreCase(id)){
					fieldlabel = "125805";
				}else if("overtime_4leave".equalsIgnoreCase(id)){
					fieldlabel = "125804";
				}else{
					fieldlabel = "523";
					column.put("showDetial","1");
					String unitType = (KQOvertimeRulesBiz.getMinimumUnit()==3 || KQOvertimeRulesBiz.getMinimumUnit()==5 ||KQOvertimeRulesBiz.getMinimumUnit()==6)?"2":"1";
					String unitTypeName = "";
					if(Util.null2String(unitType).length()>0){
						if(unitType.equals("1")){
							unitTypeName=SystemEnv.getHtmlLabelName(1925, user.getLanguage());
						}else if(unitType.equals("2")){
							unitTypeName=SystemEnv.getHtmlLabelName(391, user.getLanguage());
						}else if(unitType.equals("3")){
							unitTypeName=SystemEnv.getHtmlLabelName(18083, user.getLanguage());
						}
					}
					column.put("unit", unitTypeName);
				}
				column.put("title", SystemEnv.getHtmlLabelNames(fieldlabel, user.getLanguage()));
				column.put("dataIndex", id);
				column.put("key", id);
				column.put("rowSpan", 1);
				Map<String,Object> mapChildColumnInfo = getChildColumnsInfo(id, user);
				int childWidth = 65;
				List<Object> childColumns = (List<Object>)mapChildColumnInfo.get("childColumns");
				if(childColumns.size()>0) {//跨列width取子列的width
					column.put("children", childColumns);
					childWidth = Util.getIntValue(Util.null2String(mapChildColumnInfo.get("sumChildColumnWidth")),65);
				}
				column.put("width", childWidth+"");
				sumChildColumnWidth+=childWidth;
				lsChildColumns.add(column);
			}
		}else{
			KQReportFieldComInfo kqReportFieldComInfo = new KQReportFieldComInfo();
			while (kqReportFieldComInfo.next()){
				if(kqReportFieldComInfo.getParentid().equals(parentid)) {
					if(!kqReportFieldComInfo.getReportType().equals("month"))continue;
					column = new HashMap();
					column.put("title", SystemEnv.getHtmlLabelNames(kqReportFieldComInfo.getFieldlabel(), user.getLanguage()));
					column.put("unit", KQReportBiz.getUnitType(kqReportFieldComInfo, user));
					column.put("width", Util.getIntValue(kqReportFieldComInfo.getWidth()));
					column.put("dataIndex", kqReportFieldComInfo.getFieldname());
					column.put("key", kqReportFieldComInfo.getFieldname());
					column.put("rowSpan", 1);
					column.put("colSpan", 1);
					column.put("showDetial",kqReportFieldComInfo.getShowDetial());
					sumChildColumnWidth+=Util.getIntValue(kqReportFieldComInfo.getWidth());
					lsChildColumns.add(column);
				}
			}
		}
		returnMap.put("childColumns",lsChildColumns);
		returnMap.put("sumChildColumnWidth",sumChildColumnWidth);
		return returnMap;
	}

	private List<Object>  getCascadeKeyColumnsInfo(String cascadeKey, User user){
		List<Object> lsChildColumns = new ArrayList<>();
		if(Util.null2String(cascadeKey).length()==0){
			return lsChildColumns;
		}
		Map<String,Object> column = null;
		List<String> lsCascadeKey = Util.splitString2List(cascadeKey,",");
		KQReportFieldComInfo kqReportFieldComInfo = new KQReportFieldComInfo();
		for(int i=0;i<lsCascadeKey.size();i++){
			kqReportFieldComInfo.setTofirstRow();
			while (kqReportFieldComInfo.next()) {
				if(!kqReportFieldComInfo.getReportType().equals("month"))continue;
				if (kqReportFieldComInfo.getFieldname().equals(lsCascadeKey.get(i))){
					column = new HashMap();
					column.put("title", SystemEnv.getHtmlLabelNames(kqReportFieldComInfo.getFieldlabel(), user.getLanguage()));
					column.put("unit", KQReportBiz.getUnitType(kqReportFieldComInfo, user));
					column.put("width", Util.getIntValue(kqReportFieldComInfo.getWidth()));
					column.put("dataIndex", kqReportFieldComInfo.getFieldname());
					column.put("key", kqReportFieldComInfo.getFieldname());
					column.put("rowSpan", 1);
					column.put("colSpan", 1);
					column.put("showDetial",kqReportFieldComInfo.getShowDetial());
					column.put("isSystem", kqReportFieldComInfo.getIsSystem());
					lsChildColumns.add(column);
				}
			}
		}
		return lsChildColumns;
	}

	/**
	 * 	 * 关联流程计算加班时长  加班时长减去调休

	 */
	public double getOvertimeCount(String userid , String fromDate, String toDate, double leaveTime, String leaveType){
		BigDecimal bigDecimal = new BigDecimal(Double.toString(leaveTime));

		KQWorkTime kqWorkTime = new KQWorkTime();
		String propValue = Util.null2String(Prop.getPropValue("Others","wfTableName"));
		if("".equals(propValue)){
			return leaveTime;
		}
		String sql = " select * from " +propValue +" where jbksrq >= ? and jbksrq >= ? and sqr = ?";
		RecordSet rs = new RecordSet();
		rs.executeQuery(sql,fromDate, toDate, userid);

		while (rs.next()) {
			String workdate = rs.getString("jbksrq");
			boolean workDay = kqWorkTime.isWorkDay(userid, workdate);
			boolean isHoliday = KQHolidaySetBiz.isHoliday(userid,workdate);
			double dxsc = rs.getDouble("dxsc");
			dxsc = (double)Math.round(dxsc*100)/100;
			BigDecimal bigDecimalDxsc = new BigDecimal(Double.toString(dxsc));
			if(workDay){
				if("1".equals(leaveType)){
					bigDecimal.subtract(bigDecimalDxsc);
				}
			}else if(isHoliday ){
				if("2".equals(leaveType)){
					bigDecimal.subtract(bigDecimalDxsc);
				}

			}else {
				if("3".equals(leaveType)){
					bigDecimal.subtract(bigDecimalDxsc);
				}

			}
		}
		return bigDecimal.doubleValue();
	}

	@Override
	public BizLogContext getLogContext() {
		return null;
	}

	public static void main(String[] args) {
		String s1 = String.format("%.2f", 1.2);
		System.out.println(s1);

	}

	/**
	 * 找到人员入职日期
	 *
	 * @return
	 */
	public static String findRzrq(String jbr) {
		RecordSet rs = new RecordSet();
		String sql = "select id,companystartdate from hrmresource where id ="+jbr;
		rs.executeQuery(sql);
		String companystartdate = "";
		if (rs.next()) {
			companystartdate = Util.null2String(rs.getString("companystartdate"));
		}
		return companystartdate;
	}


	/**
	 * 找到人员离职日期
	 *
	 * @return
	 */
	public static String findLzrq(String jbr) {
		RecordSet rs = new RecordSet();
		String sql = "select changedate from HrmStatusHistory where RESOURCEID ="+jbr+" and type_n = 5";
		rs.executeQuery(sql);
		String changedate = "";
		if (rs.next()) {
			changedate = Util.null2String(rs.getString("changedate"));
		}
		return changedate;
	}

	/**
	 * 获取入离职缺勤小时
	 * @param dateOri
	 * @param userId
	 * @return
	 */
	public String getXcqts(String dateOri,String userId){
		//需出勤天数
		Integer xcqtsAll = 0;
		//获取员工的入职日期
        String rzrq = findRzrq(userId);
		//获取员工的离职日期
        String lzrq = findLzrq(userId);
		String dateOriSub = dateOri.substring(0,7);
		//本月第一天
		String fromDate = TimeUtil.getMonthBeginDay(dateOri);
		//本月最后一天
		String toDate = TimeUtil.getMonthEndDay(dateOri);
		//入职日期处理
        if(StringUtils.isNotBlank(rzrq)){
			String rzrqSub = rzrq.substring(0,7);
            if(rzrqSub.equals(dateOriSub)){
            	//入职日期在当月
				Set<String> all = new HashSet<>();
				all.add(fromDate);
				List<String> back = days(fromDate,rzrq);
				all.addAll(back);
				for (String date : all) {
					Integer zcgzts = 0;
					KQHolidaySetComInfo holidaySetComInfo = new KQHolidaySetComInfo();
					String changeTypeNew = holidaySetComInfo.getChangeType("1", date);
//					if("2".equals(changeTypeNew)){
//						//调配工作日计正常工作天数
//						zcgzts = 1;
//					}
					if(DateUtil.getWeek(date) != 6 && DateUtil.getWeek(date) != 7){
						//不为周六周日
						if(!"3".equals(changeTypeNew)){
							//不为休息日
							zcgzts = 1;
						}
					}

					xcqtsAll = xcqtsAll+zcgzts;
				}

			}
		}
        //离职日期处理
		if(StringUtils.isNotBlank(lzrq)){
			String lzrqSub = lzrq.substring(0,7);
			if(lzrqSub.equals(dateOriSub)){
				//离职日期在当月
				Set<String> all = new HashSet<>();
				List<String> back = days(lzrq,toDate);
				all.addAll(back);
				all.add(toDate);
				for (String date : all) {
					Integer zcgzts = 0;
					KQHolidaySetComInfo holidaySetComInfo = new KQHolidaySetComInfo();
					String changeTypeNew = holidaySetComInfo.getChangeType("1", date);
//					if("2".equals(changeTypeNew)){
//						//调配工作日计正常工作天数
//						zcgzts = 1;
//					}
					if(DateUtil.getWeek(date) != 6 && DateUtil.getWeek(date) != 7){
						//不为周六周日
						if(!"3".equals(changeTypeNew)){
							//不为休息日
							zcgzts = 1;
						}
					}

					xcqtsAll = xcqtsAll+zcgzts;
				}

			}
		}
		Integer newXcqTime = xcqtsAll * 8;
		new BaseBean().writeLog(userId+"newXcqTime"+newXcqTime);
		return String.valueOf(newXcqTime);
	}

	/**
	 * 获取两个日期之间所有的日期
	 * @param date1
	 * @param date2
	 * @return
	 */
	public static ArrayList days(String date1, String date2) {
		String dateFormat = "yyyy-MM-dd";
		SimpleDateFormat format = new SimpleDateFormat(dateFormat);
		ArrayList L = new ArrayList();
		if (date1.equals(date2)) {
			System.out.println("两个日期相等!");
			return L;
		}

		String tmp;
		if (date1.compareTo(date2) > 0) { // 确保 date1的日期不晚于date2
			tmp = date1;
			date1 = date2;
			date2 = tmp;
		}

		tmp = format.format(str2Date(date1).getTime() + 3600 * 24 * 1000);

		int num = 0;
		while (tmp.compareTo(date2) < 0) {
			L.add(tmp);
			num++;
			tmp = format.format(str2Date(tmp).getTime() + 3600 * 24 * 1000);
		}

		if (num == 0)
			System.out.println("两个日期相邻!");
		return L;
	}

	private static Date str2Date(String str) {
		String dateFormat = "yyyy-MM-dd";
		SimpleDateFormat format = new SimpleDateFormat(dateFormat);
		if (str == null)
			return null;

		try {
			return format.parse(str);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return null;
	}

}
