package com.engine.kq.cmd.shiftschedule;

import com.alibaba.fastjson.JSONObject;
import com.engine.common.biz.AbstractCommonCommand;
import com.engine.common.biz.SimpleBizLogger;
import com.engine.common.entity.BizLogContext;
import com.engine.core.interceptor.CommandContext;
import com.engine.kq.bean.KqShiftscheduleOperateLog;
import com.engine.kq.biz.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import weaver.conn.BatchRecordSet;
import weaver.conn.RecordSet;
import weaver.file.ImageFileManager;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.hrm.HrmUserVarify;
import weaver.hrm.Reminder.KQAutoCardTask;
import weaver.hrm.User;
import weaver.hrm.resource.ResourceComInfo;
import weaver.systeminfo.SystemEnv;

import java.text.SimpleDateFormat;
import java.util.*;

public class ImportExcelCmd extends AbstractCommonCommand<Map<String, Object>> {
  private String today = weaver.common.DateUtil.getCurrentDate();
  private SimpleBizLogger logger;

  // 添加日志功能
  private  List<List<Object>> operateLogParams = new ArrayList<>();
  // 操作类型  操作人  考勤组名称+id  被修改人  被修改班次日期  被修改班次+id  排班设置行id
  private List<Object> operateLogParam = null;

  public ImportExcelCmd(Map<String, Object> params, User user) {
    this.user = user;
    this.params = params;
    this.logger = new SimpleBizLogger();
  }

  @Override
  public Map<String, Object> execute(CommandContext commandContext) {
    Map<String, Object> retmap = new HashMap<String, Object>();
    RecordSet rs = new RecordSet();
    BatchRecordSet bRs = new BatchRecordSet();
    String sql = "";
    try {
      //必要的权限判断
      if (!HrmUserVarify.checkUserRight("HrmKQGroup:Add", user)) {
        retmap.put("status", "-1");
        retmap.put("message", SystemEnv.getHtmlLabelName(2012, user.getLanguage()));
        return retmap;
      }
      ResourceComInfo resourceComInfo = new ResourceComInfo();
      KQGroupComInfo kqGroupComInfo = new KQGroupComInfo();
      KQGroupBiz kqGroupBiz = new KQGroupBiz();
      KQShiftManagementComInfo kqShiftManagementComInfo = new KQShiftManagementComInfo();

      String groupId = Util.null2String(params.get("groupId"));
      List<String> managerRoleUserIdList = new KqShiftScheduleRoleBiz().getUfKqShiftScheduleRoleValue(""+user.getUID(), groupId);

      String filename = Util.null2String(params.get("filename"));

      List lsColumn = null;
      List<List<Object>> paramInsert = new ArrayList<List<Object>>();
      List<List<Object>> paramUpdate = new ArrayList<List<Object>>();
      List<Object> param = null;
      List<List<Object>> lsFormatParams = new ArrayList<>();
      List<Object> formatParams = null;

      List<List<Object>> delValidateParams = new ArrayList<>();
      List<Object> validateParams = null;

      List<List<Object>> lscheckSerialParams = new ArrayList<>();
      List<Object> checkSerialParams = null;

      Map<String,Object> mapShiftSchedule = new HashMap<>();
      Map<String,String> mapSerial = new HashMap<>();
      Map<String,String> mapGroupMembersWorkCode = new HashMap<>();
      Map<String,String> mapGroupMembersLastName = new HashMap<>();
      List<String> serialids=Util.splitString2List(kqGroupComInfo.getSerialids(groupId),",");
      if(!serialids.contains("-1"))serialids.add("-1");
      for(String serialid : serialids){
        if(serialid.equals("-1")){
          String restLabel = SystemEnv.getHtmlLabelName(26593, user.getLanguage()).trim();
          mapSerial.put(restLabel,serialid);
        }else{
          mapSerial.put(Util.formatMultiLang(kqShiftManagementComInfo.getSerial(serialid),""+user.getLanguage()),serialid);
        }
      }

      Map<String,Object> params = new HashMap<>();
      params.put("groupId",groupId);
      params.put("isNoAccount","1");
      List<String> kqGroupMembers = kqGroupBiz.getGroupMembers(params);

      KqMembersValidateBiz kqMembersValidateBiz = new KqMembersValidateBiz();

      Map<String, List<String>> validateResult = new KqMembersValidateBiz(params, user).getKqMembersValidateList();

      List<String> excludeidList = new ArrayList<>();
      KqShiftscheduleOperateLog bean = null;
      boolean needChange = false;
      ResourceComInfo rci = new ResourceComInfo();
      if(!Util.null2String(kqGroupComInfo.getExcludecount(groupId)).equals("1")) {
        String excludeid = Util.null2String(kqGroupComInfo.getExcludeid(groupId));
        if (excludeid.length() > 0) {
          excludeidList = Util.splitString2List(excludeid,",");
        }
      }
      for(String resourceid : kqGroupMembers){
		  // 排除无需考勤人员
        if(excludeidList != null && excludeidList.contains(resourceid)) {
          continue;
        }
        String workCode = Util.formatMultiLang(Util.null2String(resourceComInfo.getWorkcode(resourceid)),""+user.getLanguage());
        String lastName = Util.formatMultiLang(Util.null2String(resourceComInfo.getLastname(resourceid)),""+user.getLanguage());
        if(workCode.length()>0){
          mapGroupMembersWorkCode.put(workCode,resourceid);
        }else if(lastName.length()>0){
          mapGroupMembersLastName.put(lastName,resourceid);
        }
      }

      sql = " select id, resourceid, kqdate, serialid from kq_shiftschedule where groupid=? and (isdelete is null or isdelete <> '1') ";
      rs.executeQuery(sql,groupId);
      while(rs.next()){
        mapShiftSchedule.put(rs.getString("kqdate")+"|"+rs.getString("resourceid"),Util.null2String(rs.getString("id"))+"#"+Util.null2String(rs.getString("serialid")));
      }
      String ssyfDate = "";
      List<String> lsGroupMembers = new ArrayList<>();
      Map<String,Object> mapShiftScheduleOri = new HashMap<>();
      Set<String> unSavePersonAll = new HashSet<>();

      int firstRow = 6;
      int firstCol = 3;
      ImageFileManager manager = new ImageFileManager();
      manager.getImageFileInfoById(Util.getIntValue(filename));
      //Workbook workbook = WorkbookFactory.create(manager.getInputStream());
      Workbook workbook = new XSSFWorkbook(manager.getInputStream());
      for (int idx = 0; idx < workbook.getNumberOfSheets(); idx++) {
        Sheet sheet = workbook.getSheetAt(idx);
        if(sheet.getSheetName().indexOf("constraintDataSheet")>-1)continue;
        Row row = null;
        Cell cell = null;

        row = sheet.getRow(firstRow);
        lsColumn = new ArrayList();
        for (int cellIndex = firstCol; cellIndex < row.getLastCellNum(); cellIndex++) {
          cell = row.getCell((short) cellIndex);
          lsColumn.add(Util.null2String(getCellValue(cell)).trim());
        }
        firstRow++;

        for (int i = firstRow; firstRow<=sheet.getLastRowNum() && i <= sheet.getLastRowNum(); i++) {
          row = sheet.getRow(i);
          if (row == null) {
            continue;
          }
          String workCode = Util.null2String(row.getCell(1));//编号
          String lastName = Util.null2String(row.getCell(2));//姓名
          if(workCode.length()==0 && lastName.length()==0){
            continue;
          }
          String resourceId = Util.null2String(mapGroupMembersWorkCode.get(workCode));//人员id
          if(resourceId.length()==0){
            resourceId = Util.null2String(mapGroupMembersLastName.get(lastName));//人员id
          }
          if(resourceId.length()==0)continue;
          if(!managerRoleUserIdList.isEmpty()) {
            if(!managerRoleUserIdList.contains(resourceId)) continue;
          }
          lsGroupMembers.add(resourceId);
          for (int cellIndex = firstCol; cellIndex < row.getLastCellNum(); cellIndex++) {
            cell = row.getCell((short) cellIndex);
            if (cell == null) continue;
            String serialId = "";
            String serialName = Util.null2String(getCellValue(cell)).trim();
            if(serialName.length()==0)continue;
            String kqdate = Util.null2String(lsColumn.get(cellIndex-firstCol));
			kqdate = kqdate.trim().replace(String.valueOf((char)10), "#");
            kqdate = kqdate.split("#")[0];
            if(!SaveShiftScheduleCmd.isDate(kqdate)) {
              continue;
            }
            //if(weaver.common.DateUtil.timeInterval(kqdate,today)>0)continue;//今天之前的无需处理
            serialId =  Util.null2String(mapSerial.get(serialName));
            if(serialId.length()==0)continue;

            String shiftScheduleValue = Util.null2String(mapShiftSchedule.get(kqdate+"|"+resourceId));
            String id = "";
            String oldSerialid = "";
            if(!"".equals(shiftScheduleValue)) {
              id = shiftScheduleValue.split("#")[0];
              oldSerialid = shiftScheduleValue.split("#")[1];
            }

            // 如果超过了有效期，就不存储数据
            List<String> validateList = validateResult.get(resourceId);
            boolean isInRange = kqMembersValidateBiz.isInRange(kqdate, validateList);
            if(!isInRange && oldSerialid.length() > 0) {
              validateParams = new ArrayList<>();
              validateParams.add(resourceId);
              validateParams.add(kqdate);
              delValidateParams.add(validateParams);
              lsFormatParams.add(validateParams);
              bean = new KqShiftscheduleOperateLog(2,user.getLastname(),kqGroupComInfo.getGroupname(groupId)+"(id:"+groupId+")",rci.getLastname(resourceId),
                      kqdate, ("-1".equals(oldSerialid) ? SystemEnv.getHtmlLabelName(26593, user.getLanguage()) : kqShiftManagementComInfo.getSerial(oldSerialid)+"(id:"+oldSerialid+")"), id, "",user.getLanguage());
              addLog(bean);
            }

            if(id.length()>0 && !oldSerialid.equals(serialId) && oldSerialid.length() > 0){
              needChange = true;
              param = new ArrayList<Object>();
              param.add(serialId);
              param.add(id);
              param.add(resourceId);
              paramUpdate.add(param);
              bean = new KqShiftscheduleOperateLog(1, user.getLastname(), kqGroupComInfo.getGroupname(groupId) + "(id:" + groupId + ")", rci.getLastname(resourceId),
                      kqdate, ("-1".equals(oldSerialid) ? SystemEnv.getHtmlLabelName(26593, user.getLanguage()) : kqShiftManagementComInfo.getSerial(oldSerialid) + "(id:" + oldSerialid + ")"), id, ("-1".equals(serialId) ? SystemEnv.getHtmlLabelName(26593, user.getLanguage()) : kqShiftManagementComInfo.getSerial(serialId) + "(id:" + serialId + ")"),user.getLanguage());
              addLog(bean);
              ssyfDate = kqdate;
              mapShiftScheduleOri.put(kqdate+"|"+resourceId,serialId);
            }else{
              if(!oldSerialid.equals(serialId)) {
                needChange = true;
                param = new ArrayList<Object>();
                param.add(kqdate);
                param.add(serialId);
                param.add(resourceId);
                param.add(groupId);
                paramInsert.add(param);
                bean = new KqShiftscheduleOperateLog(0, user.getLastname(), kqGroupComInfo.getGroupname(groupId) + "(id:" + groupId + ")", rci.getLastname(resourceId),
                        kqdate, "", id, ("-1".equals(serialId) ? SystemEnv.getHtmlLabelName(26593, user.getLanguage()) : kqShiftManagementComInfo.getSerial(serialId) + "(id:" + serialId + ")"), user.getLanguage());
                addLog(bean);
                ssyfDate = kqdate;
                mapShiftScheduleOri.put(kqdate+"|"+resourceId,serialId);
              }
            }
            if(needChange) {
              formatParams = new ArrayList<>();
              formatParams.add(resourceId);
              formatParams.add(kqdate);
              lsFormatParams.add(formatParams);

              checkSerialParams = new ArrayList<>();
              checkSerialParams.add(resourceId);
              checkSerialParams.add(kqdate);
              checkSerialParams.add(serialId);
              lscheckSerialParams.add(checkSerialParams);
            }

          }
        }

        BaseBean bb = new BaseBean();
        String fromDate = TimeUtil.getMonthBeginDay(ssyfDate);
        String toDate = TimeUtil.getMonthEndDay(ssyfDate);
        List<String> lsDateNew = new ArrayList<>();
        Calendar calNew = weaver.common.DateUtil.getCalendar();
        boolean isEndNew = false;
        for(String date=fromDate; !isEndNew;) {
          if(date.equals(toDate)) isEndNew = true;
          lsDateNew.add(date);
          calNew.setTime(weaver.common.DateUtil.parseToDate(date));
          date = weaver.common.DateUtil.getDate(calNew.getTime(), 1);
        }
        Map<String,Object> mapShiftScheduleAll = new HashMap<>();
        Set<String> unSavePerson = new HashSet<>();
        //上四休二校验,生成预排班数据结构
        for(String resourceid:lsGroupMembers){
          Map<String,Object> mapShiftScheduleChild = new HashMap<>();
          String sqlChild = " select id, resourceid, kqdate,serialid from kq_shiftschedule where (isdelete is null or  isdelete <> '1') and groupid=? ";
          //有客户提下面的sql有性能问题，在表数据较多，并且该表数据改动较为频率不适合建索引，所以对mapShiftSchedule先缓存的数据范围过滤下，没必要全表缓存
          sqlChild += " and kqdate>='"+fromDate+"' and kqdate<='"+toDate+"' ";
          rs.executeQuery(sqlChild,groupId);
          while(rs.next()){
            mapShiftScheduleChild.put(rs.getString("kqdate")+"|"+rs.getString("resourceid"),rs.getString("serialid"));
          }
          //生成这个人员这个月的预生成排班
          for(int i=0;i<lsDateNew.size();i++) {
            String date = lsDateNew.get(i);
            String bcidNew = Util.null2String(mapShiftScheduleOri.get(date+"|"+resourceid));
            if(StringUtils.isNotBlank(bcidNew)){
              //本次排班预生成班次id
              mapShiftScheduleAll.put(date+"|"+resourceid,bcidNew);
            }else{
              //本次排班未预生成id，检查历史生成的班次id，若历史也没有生效班次，且当天不是节假日，则不让保存
              String bcidOld = Util.null2String(mapShiftScheduleChild.get(date+"|"+resourceid));
              if(StringUtils.isNotBlank(bcidOld)){
                mapShiftScheduleAll.put(date+"|"+resourceid,bcidOld);
              }else{
                int changeType = getChangeType(groupId, date);
                if(changeType != 1){
                  //该人员不让保存
                  unSavePerson.add(resourceid);
                }
              }
            }
          }
        }
        bb.writeLog("unSavePerson"+ JSONObject.toJSONString(unSavePerson));
        bb.writeLog("mapShiftScheduleAll"+JSONObject.toJSONString(mapShiftScheduleAll));

        // 校验是否满足上四休二规则：从当月第一个连续两天的休息班作为锚点，判断此休息班前后排班是否满足固定4天工作+2天休息的模式，
        // 如果当月存在法定节假日则默认跳过后继续排班。例如：周三为节假日，且当周一为第一天工作日，那么当周的排班结果为：周一~周二：
        // 2天工作日、周三：跳过、周四~周五：2天工作日、周六周日：2天休息班
        for(String resourceid:lsGroupMembers){
          bb.writeLog("resourceid"+resourceid);
          //遍历这个人员这个月的预生成排班
          for(int i=0;i<lsDateNew.size();i++) {
            String date = lsDateNew.get(i);
            String bcid = Util.null2String(mapShiftScheduleAll.get(date+"|"+resourceid));
            bb.writeLog("date"+date+"bcid"+bcid);
            if(StringUtils.isNotBlank(bcid)){
              //班次是否是休息班，是的话往后判断是否是两个连续休息班
              boolean checkXxb = checkXxr(bcid);
              if(checkXxb){
                //是休息班,往后找
                bb.writeLog("是休息班,往后找");
                boolean findNextXxb = true;
                Integer day = 1;
                while (findNextXxb){
                  calNew.setTime(weaver.common.DateUtil.parseToDate(date));
                  String dateNext = weaver.common.DateUtil.getDate(calNew.getTime(), day);
                  day++;
                  String bcidNext = Util.null2String(mapShiftScheduleAll.get(dateNext+"|"+resourceid));
                  bb.writeLog("dateNext"+dateNext+"day"+day+"bcidNext"+bcidNext);
                  if(StringUtils.isNotBlank(bcidNext)){
                    //找到了第一个有班次的
                    bb.writeLog("找到了第一个有班次的");
                    findNextXxb = false;
                    boolean checkXxbNext = checkXxr(bcidNext);
                    if(checkXxbNext){
                      //再往后找一个班次，该班次需要为工作日
                      //是休息班,往后找
                      bb.writeLog("也是休息班,再往后找");
                      boolean findNextXxb1 = true;
                      Integer day1 = 1;
                      while (findNextXxb1){
                        calNew.setTime(weaver.common.DateUtil.parseToDate(dateNext));
                        String dateNext1 = weaver.common.DateUtil.getDate(calNew.getTime(), day1);
                        day1++;
                        String bcidNext1 = Util.null2String(mapShiftScheduleAll.get(dateNext1+"|"+resourceid));
                        bb.writeLog("bcidNext1"+bcidNext1+"dateNext1"+dateNext1+"day1"+day1);
                        if(StringUtils.isNotBlank(bcidNext1)){
                          //不往后找了
                          boolean checkXxbNext1 = checkXxr(bcidNext1);
                          findNextXxb1 = false;
                          bb.writeLog("checkXxbNext1"+checkXxbNext1);
                          if(checkXxbNext1){
                            //存在三个连续的休息班，不符合上四休二
                            bb.writeLog("存在三个连续的休息班，不符合上四休二");
                            unSavePerson.add(resourceid);
                          }else{
                            i++;
                          }
                        }else{
                          i++;
                        }
                        //到最后一天也没找到,暂不处理
                        if(weaver.common.DateUtil.timeInterval(toDate,dateNext1)>=0){
                          findNextXxb1 = false;
                        }
                      }
                      //将date作为一个锚点，往前找并判断是否满足上四休二
                      boolean findBefore = true;
                      Integer dayB = -1;
                      //工作班数量
                      Integer gzbnum = 0;
                      //判断前面是否还有休息班标识
                      Integer xxbNo = 0;
                      while (findBefore){
                        calNew.setTime(weaver.common.DateUtil.parseToDate(date));
                        String dateBefore = weaver.common.DateUtil.getDate(calNew.getTime(), dayB);
                        dayB--;
                        String bcidBefore = Util.null2String(mapShiftScheduleAll.get(dateBefore+"|"+resourceid));
                        bb.writeLog("bcidBefore"+bcidBefore+"dateBefore"+dateBefore+"dayB"+dayB);
                        if(StringUtils.isNotBlank(bcidBefore)){
                          boolean checkXxbBefore = checkXxr(bcidBefore);
                          if(checkXxbBefore){
                            //找到了第一个休息班，判断间隔了多少个非休息班,不再找了
                            bb.writeLog("找到了第一个休息班，判断间隔了多少个非休息班,不再找了");
                            findBefore = false;
                            xxbNo = 1;
                          }else{
                            gzbnum ++;
                          }
                        }
                        //找到月初一号了，不找了
                        if(weaver.common.DateUtil.timeInterval(dateBefore,fromDate)>=0){
                          findBefore = false;
                        }
                      }
                      bb.writeLog("gzbnum"+gzbnum);
                      if(xxbNo == 1){
                        //当前锚点前面有休息班，若间隔工作班数量不为4，则不符合
                        if(gzbnum != 4){
                          bb.writeLog("当前锚点前面有休息班，若间隔工作班数量不为4，则不符合");
                          unSavePerson.add(resourceid);
                        }
                      }else{
                        //当前锚点是第一个休息班
                        if(gzbnum>4){
                          //锚点往前有超过四个工作班,不符合
                          bb.writeLog("当前锚点是第一个休息班,锚点往前有超过四个工作班,不符合");
                          unSavePerson.add(resourceid);
                        }
                      }

                    }else{
                      //找到的不是休息班，也认为不符合上四休二
                      bb.writeLog("找到的不是休息班，也认为不符合上四休二");
                      unSavePerson.add(resourceid);
                    }
                  }else{
                    i++;
                  }

                  //到最后一天也没找到,暂不处理
                  if(weaver.common.DateUtil.timeInterval(toDate,dateNext)>=0){
                    findNextXxb = false;
                  }
                }
              }
            }
          }
        }
        bb.writeLog("unSavePerson"+JSONObject.toJSONString(unSavePerson));
        unSavePersonAll.addAll(unSavePerson);

        sql = " update kq_shiftschedule set serialid=?, isDelete=0 where id = ? ";
        for (int i = 0; paramUpdate != null && i < paramUpdate.size(); i++) {
          List<Object> update_params = paramUpdate.get(i);
          String serialid = Util.null2String(update_params.get(0));
          String id = Util.null2String(update_params.get(1));
          String resourceid = Util.null2String(update_params.get(2));
          if(unSavePerson.contains(resourceid)){
            //该人员不符合规则，不生效
          }else{
            rs.executeUpdate(sql, serialid,id);
          }
        }

        sql = "insert into kq_shiftschedule (kqdate,serialid,resourceid,groupid,isDelete) values(?,?,?,?,0)";
        for (int i = 0; paramInsert != null && i < paramInsert.size(); i++) {
          List<Object> insert_params = paramInsert.get(i);
          String kqdate = Util.null2String(insert_params.get(0));
          String serialid = Util.null2String(insert_params.get(1));
          String resourceid = Util.null2String(insert_params.get(2));
          String groupid = Util.null2String(insert_params.get(3));
          if(unSavePerson.contains(resourceid)){
            //该人员不符合规则，不生效
          }else{
            rs.executeUpdate(sql, kqdate,serialid,resourceid,groupid);
          }
        }
      }

      // 删除一些有效期之后的排班
      sql = " update kq_shiftschedule set isdelete =1 where resourceid = ? and kqdate = ? and groupid = "+groupId;
      for (List<Object> delete_params : delValidateParams) {
        String resourceid = Util.null2String(delete_params.get(0));
        String kqdate = Util.null2String(delete_params.get(1));
        if(unSavePersonAll.contains(resourceid)){
          //该人员不符合规则，不生效
        }else{
          rs.executeUpdate(sql, resourceid, kqdate);
        }

      }

      //删除之前的排的班次，一天只能有一个班次
      sql = " update kq_shiftschedule set isdelete =1 where (isdelete is null or isdelete <> '1') and resourceid = ? and kqdate = ? and groupid != "+groupId;
      for (int i = 0; lsFormatParams != null && i < lsFormatParams.size(); i++) {
        List<Object> delete_params = lsFormatParams.get(i);
        String resourceid = Util.null2String(delete_params.get(0));
        String kqdate = Util.null2String(delete_params.get(1));
        if(unSavePersonAll.contains(resourceid)){
          //该人员不符合规则，不生效
        }else{
          rs.executeUpdate(sql, resourceid,kqdate);
        }
      }

      sql = " update kq_shiftschedule set isdelete =1 where (isdelete is null or isdelete <> '1') and resourceid = ? and kqdate = ? and serialid != ? ";
      for (int i = 0; lscheckSerialParams != null && i < lscheckSerialParams.size(); i++) {
        List<Object> check_params = lscheckSerialParams.get(i);
        String resourceid = Util.null2String(check_params.get(0));
        String kqdate = Util.null2String(check_params.get(1));
        String serialid = Util.null2String(check_params.get(2));
        if(unSavePersonAll.contains(resourceid)){
          //该人员不符合规则，不生效
        }else{
          rs.executeUpdate(sql, resourceid,kqdate,serialid);
        }
      }

      sql = "insert into kq_shiftscheduleoperate_log(operatetype,operator,kqgroupname,changeduser,changedshiftdate,changedshiftname,kqshiftid) values(?,?,?,?,?,?,?)";
      bRs.executeBatchSqlNew(sql, operateLogParams);

      new KQShiftScheduleComInfo().removeCache();

      //刷新报表数据
//      sql = " insert into kq_format_pool (resourceid, kqdate) values (?,?)";
//      bRs.executeBatchSql(sql, lsFormatParams);
      new KQFormatBiz().format(lsFormatParams,16);

      KQGroupComInfo kQGroupComInfo = new KQGroupComInfo();
      String auto_checkout = kQGroupComInfo.getAuto_checkout(groupId);
      String auto_checkin = kQGroupComInfo.getAuto_checkin(groupId);
      if("1".equalsIgnoreCase(auto_checkin) || "1".equalsIgnoreCase(auto_checkout)){
        //如果开启了自动打卡，保存考勤组成员之后需要格式化下缓存
        KQAutoCardTask kqAutoCardTask = new KQAutoCardTask();
        kqAutoCardTask.initAutoCardTask(groupId);
      }

      if(unSavePersonAll.size()>0){
        List<String> personNameList = new ArrayList<>();
        for(String personid:unSavePersonAll){
          personNameList.add(resourceComInfo.getLastname(personid));
        }
        retmap.put("status", "-1");
        retmap.put("message", "【"+StringUtils.join(personNameList,",")+"】上述员工当前月份排班不符合做四休二规则或存在班次为空的日期");
        return retmap;
      }

      retmap.put("status", "1");
    } catch (Exception e) {
      retmap.put("status", "-1");
      retmap.put("message", SystemEnv.getHtmlLabelName(382661, user.getLanguage()));
      writeLog(e);
    }
    return retmap;
  }

  @Override
  public BizLogContext getLogContext() {
    return null;
  }

  @Override
  public List<BizLogContext> getLogContexts() {
    return logger.getBizLogContexts();
  }

  /**
   * /**
   * 获取excel单元格值
   *
   * @param cell 要读取的单元格对象
   * @return
   */
  public String getCellValue(Cell cell) {
    String cellValue = "";
    if (cell == null)
      return "";
    switch (cell.getCellType()) {
      case BOOLEAN: // 得到Boolean对象的方法
        cellValue = String.valueOf(cell.getBooleanCellValue());
        break;
      case NUMERIC:
        if (DateUtil.isCellDateFormatted(cell)) {// 先看是否是日期格式
          SimpleDateFormat sft = new SimpleDateFormat("yyyy-MM-dd");
          cellValue = String.valueOf(sft.format(cell.getDateCellValue())); // 读取日期格式
        } else {
          cellValue = String.valueOf(new Double(cell.getNumericCellValue())); // 读取数字
          if (cellValue.endsWith(".0"))
            cellValue = cellValue.substring(0, cellValue.indexOf("."));
        }
        break;
      case FORMULA: // 读取公式
        cellValue = cell.getCellFormula();
        break;
      case STRING: // 读取String
        cellValue = cell.getStringCellValue();
        break;
    }

    return cellValue;
  }

  private void addLog(KqShiftscheduleOperateLog bean) {
    operateLogParam = new ArrayList<>();
    operateLogParam.add(bean.getOperatetype());
    operateLogParam.add(bean.getOperator());
    operateLogParam.add(bean.getKqgroupname());
    operateLogParam.add(bean.getChangeduser());
    operateLogParam.add(bean.getChangedshiftdate());
    operateLogParam.add(bean.getChangedshiftname());
    operateLogParam.add(bean.getKqshiftid());
    operateLogParams.add(operateLogParam);
  }

  /**
   * 获取当前日期是何种类型(这里只考虑节假日设置)
   *
   * @param groupId 考勤组ID
   * @param date    指定日期
   * @return 1-节假日、2-工作日、3-休息日、-1-没有设置过数据
   */
  public static int getChangeType(String groupId, String date) {
    int changeType = -1;//默认没有设置过节假日(节假日设置的类型：1-公众假日、2-调配工作日、3-调配休息日)
    KQHolidaySetComInfo kqHolidaySetComInfo = new KQHolidaySetComInfo();
    String tmpChangetype = kqHolidaySetComInfo.getChangeType(groupId, date);
    if(null != tmpChangetype){
      changeType = Util.getIntValue(tmpChangetype, -1);
    }

    return changeType;
  }

  /**
   * 判断是否是休息班
   * @param serialid
   * @return
   */
  private static Boolean checkXxr(String serialid){
    RecordSet rs = new RecordSet();
    String sql = "select * from kq_ShiftManagement where id = "+serialid+" and is_rest = 1";
    rs.execute(sql);
    Boolean check = false;
    if (rs.getCounts()>0){
      check = true;
    }
    return check;
  }
}
